package MCUtils::MPL 1.01;

use parent qw(Exporter);
our @EXPORT	= qw(create_mpl_item output_mpl);
#our @EXPORT_OK	= qw();

use diagnostics;
use v5.14;
use utf8;
use strict;
use warnings;

use Tie::File;
use MCUtils::Misc;

#binmode STDOUT, ':encoding(UTF-8)';
#binmode STDERR, ':encoding(UTF-8)';

sub create_mpl_item {
    my ($required_keys, $lref) = @_;
    my @a = ();

    return @a unless $lref;

    for my $cellsref (@$lref) {
	# Only output the item if it contains the requisite fields (eg. filename and media type).
	for (keys $required_keys) {
		if (! grep /$_/i, keys $cellsref) {
		verbose "create_mpl_item: Missing required field $_: skipping: ", $cellsref->{'Filename'};
		return @a;
	    }
	}

	my @tmp = ();
	for (keys %$cellsref) {
	    if ($cellsref->{$_} and $cellsref->{$_} ne '' and $cellsref->{$_} ne '-') {
		my $val = $cellsref->{$_};
		$val =~ s/\&(?!amp;|quot;)/&amp;/g;
		$val =~ s/</&lt;/g;
		$val =~ s/>/&gt;/g;
		push @tmp, join '', '      <Field Name="', $_, '">', $val, "</Field>\n";
	    }
	}

	if (scalar @tmp <= scalar keys $required_keys) {
	    verbose "create_mpl_item: No fields to output: skipping: ", $cellsref->{'Filename'};
	    return @a;
	}
	if (@tmp) {
	    push @a, "   <Item>\n";
	    push @a, @tmp;
	    push @a, "   </Item>\n";
	}
    }

    return @a;
}

sub output_mpl {
    my ($progstr, $filepath, $append, $mplref) = @_;
    my ($fh, $openmode, $do_preamble) = (\*STDOUT, '>', 1);

    if ($filepath ne '-') {
	# in append mode, remove the closing MPL tag in an existing file
	if ($append and -e $filepath and ! -z _) {
	    my @lines;

	    tie(@lines, 'Tie::File', $filepath) or
		die "Can't update file $filepath\nStopped";
	    delete $lines[-1] if $lines[-1] =~ /^<\/MPL>/;
	    untie @lines;
	    $openmode = '>>';
	    $do_preamble = 0;
	}

	#my $encoding = ' :utf8';
	my $encoding = '';
	open($fh, $openmode . $encoding, $filepath) or
	    die "Failed to open $filepath: $!\nStopped";
    }

    if ($do_preamble) {
	print $fh qq/<?xml version="0.0" encoding="UTF-8" standalone="yes" ?>\n/,
		   qq/<MPL Version="2.0" Title="Created from $progstr">\n/;
    }

    print $fh @$mplref,
	       "</MPL>\n";
    close $fh;
}
