# Copies and Merges field values for the fields specified with -f.  The values
# are collected and grouped based on a keyfield column that is specified with
# --args | -a.  All selected tracks in the view are scanned, and for each unique
# keyfield value, the specified -f fields are collected and joined. The merged
# values are semicolon separated, and null values are omitted.

package CopyAndMerge 1.00;

use v5.16;
use strict;
use warnings;

$DB::single = 1;

our %callbacks = (
    pre     => \&do_preprocessing,
    perfile => \&do_perfile,
    post    => undef,
);

my %groups;

# Pre-processing callback function
sub do_preprocessing {
    my ($keyfield, $tracksref, $fieldsref) = @_;

    $keyfield or
	die "Required grouping field missing.  Supply the name of a field using -a <field name>.\nStopped";

    $keyfield = lc $keyfield;

    /^${keyfield}$/i ~~ %{$tracksref->[0]} or
	die "The view is missing the column '$keyfield', which defines the file grouping\nStopped";

    # For each of the user-specified fields, loop through the tracks table, 
    # collecting the field values in groups by field.  This creates a 3-level
    # hash in the form of <keyfield value> <field> <cell value>.
    for my $field (@$fieldsref) {
	for my $row (@$tracksref) {
	    if ($row->{$field} ne '') {
		$groups{$row->{$keyfield}}{$field}{$_}++   for split /\s*;\s*/, $row->{$field};
	    }
	}
    }

    # run through the groups to create a copy / merged field
    foreach my $keyfield (keys %groups) {
	foreach my $field (keys $groups{$keyfield}) {
	    $groups{$keyfield}{$field} = join '; ', keys $groups{$keyfield}{$field};
	}
    }

    return undef;
}

# Per-row callback function
sub do_perfile {
    my ($keyfield, $field, $row) = @_;

    return $groups{$row->{lc $keyfield}}{$field} // undef;
}

1;
