# Reads an the IDV3 Frame from a file.  The tag read is supplied via -a <tagname>
# or the field name will be read as a value from the field specified via -f.
#
# Requires MP3::Tag.  For Windows, install with the command line:
#
#     ppm install ppm install MP3-Tag

package GetIDV3Tag 1.01;

use v5.16;
use strict;
use warnings;
use MP3::Tag;

our %callbacks = (
    pre     => undef,
    perfile => \&do_perfile,
    post    => undef,
);

$DB::single = 1;

# Per-row callback function
sub do_perfile {
    my ($tag, $field, $row) = @_;

    my $mp3 = MP3::Tag->new($row->{'filename'});
    $mp3->get_tags;
    if (exists $mp3->{ID3v2}) {
	$tag ||= $row->{$field};
	$tag  or die "Please supply a tag name either via -a or in the contents of the field '$field'.\nStopped";
	my ($info,$name) = $mp3->{ID3v2}->get_frame($tag);
	main::debug "Name: $name: Info: $info";
	return $info ? $info : undef;
    }
}

1;
