# calls mediainfo executable to obtain the number of audio tracks and subtitle text

package GetMediaInfo 1.02;

use v5.16;
use strict;
use warnings;

our %callbacks = (
    pre     => undef,
    perfile => \&do_perfile,
    post    => undef,
);

my %options = (
    var => { 
	exe_path_win	=> 'mediainfo.exe',
	exe_path_mac	=> 'mediainfo',
    },
);

$DB::single = 1;

main::set_scriptlet_opts(\%options);

my $exe = $^O eq 'MSWin32' ? $options{'var'}{'exe_path_win'} : $options{'var'}{'exe_path_mac'};

# Per-row callback function
sub do_perfile {
    my ($argstr, $field, $row) = @_;

    -e $exe or die "Command '$exe' cannot be found\nStopped";
    -x $exe or die "command '$exe' is not executable.\nStopped";

    my $results = qx($exe, "--full", "$row->{'filename'}");

    my ($nstreams_audio) = ($results =~ /^Count of audio streams\s+:\s+(\d+)$/ms);
    my ($nstreams_text) = ($results =~ /^Count of text streams\s+:\s+(\d+)$/ms);

    $nstreams_audio ||= 0;
    $nstreams_text ||= 0;
    main::debug("Results: $nstreams_audio, $nstreams_text");

    return "audio streams: $nstreams_audio; text streams: $nstreams_text";
}

1;
