# Returns the list of playlists that include the selected files.  The full path of the each
# playlists is returned.  If a file exists in more than a single playlist, a semicolon-separated
# list of playlists is returned.

package WhichPlaylists 1.02;

use v5.16;
use strict;
use warnings;

our %callbacks = (
    pre     => \&do_preprocessing,
    perfile => \&do_perfile,
    post    => undef,
);

$DB::single = 1;

my %options = (
    var => { 
	ignored_playlists_re	=> undef,		# RE defining playlists to ignore
    },
);

main::set_scriptlet_opts(\%options);

my ($mcws, $playlists, %files);

sub do_preprocessing {
    my ($argstr, $tracksref, $fieldsref) = @_;

    exists $main::opts{'nomcws'} and 
	die "The option --nomcws cannot be used with this scriplet (MCWS is required).\nStopped";

    my $ignored_re;
    if (exists $options{'var'}{'ignored_playlists_re'}) {
	$ignored_re = qr/$options{'var'}{'ignored_playlists_re'}/;
    }

    $mcws = MCUtils::MCWS->connect(%main::mcwsparams);

    $playlists = $mcws->get_playlists(qw/Playlist/) or
	die "No playlists returned from get_playlists() call.\nStopped";

    for my $path (keys %$playlists) {
	main::verbose "Playlists found: ", $playlists->{$path}{'id'}, ' = ', $playlists->{$path}{'name'}, ': ', $path;
	if ($ignored_re and $path =~ /$ignored_re/) {
	    main::verbose "\tSkipping ignored playlist: ", $path;
	    next;
	}

	my $contents = $mcws->get_playlist_contents($playlists->{$path}{'id'}) or
	    main::verbose 'empty playlist: ', $path;
	for my $file (@$contents) {
	    main::debug "\tFile: ", $file;
	    push @{$files{$file}}, $path;
	}
    }

    return undef;
}

sub do_perfile {
    my ($argstr, $cellval, $row) = @_;

    return join '; ', @{$files{$row->{'filename'}}}	if exists $files{$row->{'filename'}};

    return undef;
}

1;
